from .services.apigateway import APIGateway
from .services.apigatewayv2 import APIGatewayV2
from .services.cloudformation import CloudFormation
from .services.cloudwatch import CloudWatch
from .services.cloudwatchlogs import CloudWatchLogs
from .services.dynamodb import DynamoDB
from .services.eventbridge import EventBridge
from .services.iot import IoT
from .services.lambda_aws import Lambda
from .services.resourcegroupstaggpingapi import ResourceGroupsTaggingAPI
from .services.s3 import S3
from .services.secretsmanager import SecretsManager
from .services.sns import SNS
from .services.sqs import SQS
from .services.sso import SSO
from .services.ssooidc import SSO_OIDC
from .services.sts import STS